/**
 * 
 */
package de.fhtw.dataprofiler.dbconnectivity.reverseengineerer;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.IPath;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.ecore.xmi.impl.XMLResourceImpl;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;

import de.fhtw.dataprofiler.profiling.analyses.DataProfilerImpl;
import de.fhtw.dataprofiler.sqlmodel.constraints.ConstraintsFactory;
import de.fhtw.dataprofiler.sqlmodel.constraints.ConstraintsPackage;
import de.fhtw.dataprofiler.sqlmodel.constraints.ForeignKey;
import de.fhtw.dataprofiler.sqlmodel.constraints.PrimaryKey;
import de.fhtw.dataprofiler.sqlmodel.schema.Schema;
import de.fhtw.dataprofiler.sqlmodel.schema.SchemaFactory;
import de.fhtw.dataprofiler.sqlmodel.schema.SchemaPackage;
import de.fhtw.dataprofiler.sqlmodel.tables.BaseTable;
import de.fhtw.dataprofiler.sqlmodel.tables.Column;
import de.fhtw.dataprofiler.sqlmodel.tables.PersistentTable;
import de.fhtw.dataprofiler.sqlmodel.tables.Table;
import de.fhtw.dataprofiler.sqlmodel.tables.TablesFactory;
import de.fhtw.dataprofiler.sqlmodel.tables.TablesPackage;

/**
 * Class for extracting the data model of a given database
 * 
 * @author Bjoern Knebel
 *
 */
public class ReverseEngineerer {

	//Accessors for the meta objects
	protected SchemaPackage _schemaPackage = SchemaPackage.eINSTANCE;
	protected TablesPackage _tablesPackage = TablesPackage.eINSTANCE;
	protected ConstraintsPackage _constraintsPackage = ConstraintsPackage.eINSTANCE;

	//Provides create methods for model classes
	protected SchemaFactory _schemaFactory = _schemaPackage.getSchemaFactory();
	protected TablesFactory _tablesFactory = _tablesPackage.getTablesFactory();
	protected ConstraintsFactory _constraintsFactory = _constraintsPackage.getConstraintsFactory();
	
	/**
	 * Extracts the data model, starts data profiling and saves the schema
	 * 
	 * @param connection Active database connection
	 * @param modelFilePath to be saved to
	 */
	public void startReverseEngineering(Connection connection, IPath modelFilePath) {
		
		//Retrieve Meta Data
		//		
		Schema schema = retrieveMetaData(connection);
		if (schema != null){
			
			// DATA PROFILING !!!!
			//
			new DataProfilerImpl(connection).startProfiling(schema);
			
			// Save the contents of the schema to the file system.
			//
			saveSchema(schema, modelFilePath);
		}
	}

	/**
	 * Retrieves the meta data or data model
	 * 
	 * @param connection active connection with specific database
	 * @return filled model objects
	 */
	private Schema retrieveMetaData(Connection connection) {
		try {
			MessageDialog.openInformation(Display.getCurrent().getActiveShell(), "Active connection", "Extracting data model from database:  "+ connection.getCatalog());
			
			System.out.println("retrieving meta data");
	
			DatabaseMetaData dbmd = connection.getMetaData();
	
			Schema schema = _schemaFactory.createSchema();
			schema.setName(connection.getCatalog());
			EList<Table> schemaTables = schema.getTables();
	
			// Read tables from meta data
			String[] tableTypes = { "TABLE" };
			ResultSet tablesResultSet = dbmd.getTables(null, null, null, tableTypes);
	
			while (tablesResultSet.next()) {
				// Initialize new table object
				PersistentTable persistentTable = _tablesFactory.createPersistentTable();
				persistentTable.setName(tablesResultSet.getString("TABLE_NAME"));
				schemaTables.add(persistentTable);
	
				//Read columns for current table
				EList<Column> columns = persistentTable.getColumns();
				ResultSet columnsResultSet = dbmd.getColumns(null, null, persistentTable.getName(), null);
				while (columnsResultSet.next()) {
					// Initialize new column object
					Column col = _tablesFactory.createColumn();
					col.setName(columnsResultSet.getString("COLUMN_NAME"));
					col.setType(columnsResultSet.getString("TYPE_NAME").toLowerCase());
					columns.add(col);
				}
	
				//Get primary key columns for current table
				PrimaryKey primaryKey = _constraintsFactory.createPrimaryKey();
				ResultSet pkResultSet = dbmd.getPrimaryKeys(null, null, persistentTable.getName());
				EList<Column> pkCols = primaryKey.getMembers();
				while (pkResultSet.next()) {
					primaryKey.setName(pkResultSet.getString("PK_NAME"));
	
					//Adds all primary key columns
					for (Column col : columns) {
						if (col.getName().equals(pkResultSet.getString("COLUMN_NAME"))) {
							pkCols.add(col);
						}
					}
				}
				//Add primary key object
				persistentTable.getConstraints().add(primaryKey);
	
			}
	
			//FOREIGN KEYS
			for (Table table : schemaTables) {
				ResultSet foreignKeyResultSet = dbmd.getImportedKeys(null, null, table.getName());
				while (foreignKeyResultSet.next()) {
					ForeignKey fk = _constraintsFactory.createForeignKey();
					fk.setName(foreignKeyResultSet.getString("FK_NAME"));
	
					//Set referenced table and columns
					for (Table refTable : schemaTables) {
						if (refTable.getName().equals(foreignKeyResultSet.getString("PKTABLE_NAME"))) {
							fk.setReferencedTable((BaseTable) refTable);
							for (Column refColumn : refTable.getColumns()) {
								if (refColumn.getName().equals(foreignKeyResultSet.getString("PKCOLUMN_NAME"))) {
									fk.getReferencedMembers().add(refColumn);
								}
							}
						}
					}
	
					//Add foreign key columns
					EList<Column> fkCols = fk.getMembers();
					for (Column col : table.getColumns()) {
						if (col.getName().equals(foreignKeyResultSet.getString("FKCOLUMN_NAME"))) {
							fkCols.add(col);
						}
					}
					//Add foreign key object
					((PersistentTable) table).getConstraints().add(fk);
				}
			}
			
			return schema;

		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}


	/**
	 * Saves the given scheme object to a xml file
	 * 
	 * @param schema - object tree to save?
	 * @param modelFilePath - where to save?
	 */
	private void saveSchema(Schema schema, IPath modelFilePath){
			
		try {
	
			// Get the URI of the model file.
			//
			URI fileURI = URI.createPlatformResourceURI(modelFilePath.toString(), true);
	
			// Create a resource for this file.
			//
			Resource resource = new XMLResourceImpl(fileURI);
	
			// Add the initial model object to the contents.
			//
			if (schema != null) {
				resource.getContents().add(schema);
			}
	
			// Save the contents of the resource to the file system.
			//
			Map<Object, Object> options = new HashMap<Object, Object>();
			//do not encode escape characters
			options.put(XMLResource.OPTION_SKIP_ESCAPE,true);
			//encoding style UTF-8
			options.put(XMLResource.OPTION_ENCODING,"UTF-8");
			resource.save(options); //XML serialization
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
