package de.fhtw.dataprofiler.sqlmodel.diagram.profile.views;

import org.eclipse.gef.EditPart;
import org.eclipse.gmf.runtime.notation.View;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.ViewPart;

import de.fhtw.dataprofiler.sqlmodel.diagram.edit.parts.Column2EditPart;
import de.fhtw.dataprofiler.sqlmodel.diagram.edit.parts.ColumnEditPart;
import de.fhtw.dataprofiler.sqlmodel.diagram.edit.parts.PersistentTableColumnsCompartmentEditPart;
import de.fhtw.dataprofiler.sqlmodel.diagram.edit.parts.PersistentTableEditPart;
import de.fhtw.dataprofiler.sqlmodel.diagram.edit.parts.PersistentTableNameEditPart;
import de.fhtw.dataprofiler.sqlmodel.diagram.edit.parts.PrimaryKeyEditPart;
import de.fhtw.dataprofiler.sqlmodel.diagram.edit.parts.PrimaryKeyPrimaryKeyColumnsCompartmentEditPart;
import de.fhtw.dataprofiler.sqlmodel.diagram.edit.parts.WrapLabelEditPart;
import de.fhtw.dataprofiler.sqlmodel.diagram.part.SqlmodelVisualIDRegistry;
import de.fhtw.dataprofiler.sqlmodel.diagram.profile.ProfilerUtils;
import de.fhtw.dataprofiler.sqlmodel.tables.Column;
import de.fhtw.dataprofiler.sqlmodel.tables.PersistentTable;

/**
 * This class implements an own view for the table profile. The view
 * shows data obtained from the model. It responds to selection changes.
 *
 * @author Bjoern Knebel
 */

public class TableProfileView extends ViewPart implements ISelectionListener {
	private TableViewer viewer;

	/*
	 * The content provider class is responsible for providing objects to the
	 * view. It returns an array of columns from the selected table
	 */

	class ViewContentProvider implements IStructuredContentProvider {
		PersistentTable selTable;

		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
			selTable = (PersistentTable) newInput;
		}

		public void dispose() {
		}

		public Object[] getElements(Object parent) {
			return selTable.getColumns().toArray();
		}
	}

	/**
	 * The label provider defines how the model objects should be presented in the view.
	 *
	 */
	class ViewLabelProvider extends LabelProvider implements ITableLabelProvider {
		public String getColumnText(Object obj, int index) {
			switch (index) {
			case 0:
				return (obj instanceof Column) ? ((Column) obj).getName() : null;
			case 1:
				return (obj instanceof Column) ? String.valueOf(((Column) obj).getTotalValues()) : null;
			case 2:
				return (obj instanceof Column) ? String.valueOf(((Column) obj).getDistinctValues()) : null;
			case 3:
				return (obj instanceof Column) ? ProfilerUtils.getPercentage(((Column) obj).getTotalValues(),
						((Column) obj).getDistinctValues()) : null;
			case 4:
				return (obj instanceof Column) ? String.valueOf(((Column) obj).getNullValues()) : null;
			case 5:
				return (obj instanceof Column) ? ProfilerUtils.getPercentage(((Column) obj).getTotalValues(),
						((Column) obj).getNullValues()) : null;
			case 6:
				return (obj instanceof Column) ? ((Column) obj).getMinValue() : null;
			case 7:
				return (obj instanceof Column) ? ((Column) obj).getMaxValue() : null;
			case 8:
				return (obj instanceof Column) ? String.valueOf(((Column) obj).getMinLength()) : null;
			case 9:
				return (obj instanceof Column) ? String.valueOf(((Column) obj).getMaxLength()) : null;
			}
			return null;
		}

		public Image getColumnImage(Object obj, int index) {
			// return
			// PlatformUI.getWorkbench().getSharedImages().getImage(ISharedImages.IMG_OBJ_ELEMENT);
			return null;
		}
	}

	/**
	 * The constructor.
	 */
	public TableProfileView() {
	}

	/**
	 * This is a callback that will allow us to create the viewer and initialize
	 * it.
	 */
	public void createPartControl(Composite parent) {
		viewer = new TableViewer(parent, SWT.BORDER | SWT.FULL_SELECTION);
		viewer.setContentProvider(new ViewContentProvider());
		viewer.setLabelProvider(new ViewLabelProvider());

		// define layout of table
		viewer.getTable().setLinesVisible(true);
		viewer.getTable().setHeaderVisible(true);
		viewer.getTable().setBackground(parent.getDisplay().getSystemColor(SWT.COLOR_YELLOW));

		// define columns
		addColumn(120, "Column name");
		addColumn(50, "Total");
		addColumn(50, "Distinct");
		addColumn(70, "% Distinct");
		addColumn(50, "Null");
		addColumn(70, "% Null");
		addColumn(100, "Min Value");
		addColumn(100, "Max Value");
		addColumn(70, "Min Length");
		addColumn(70, "Max Length");

		// listen to selection changes
		getViewSite().getPage().addSelectionListener(this);

		// sets default model to view
		ISelection selection = getViewSite().getPage().getSelection();
		viewer.setInput(getModelFromSelection(selection));
	}

	/**
	 * Add a new column to the viewer's table
	 * 
	 * @param width
	 * @param text
	 */
	private void addColumn(int width, String text) {
		TableColumn column = new TableColumn(viewer.getTable(), SWT.NONE);
		column.setWidth(width);
		column.setText(text);
		column.setMoveable(true);
	}

	private void showMessage(String message) {

		MessageDialog.openInformation(viewer.getControl().getShell(), "Table Profile View", message);
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {

		viewer.getControl().setFocus();
	}

	/**
	 * Invoked once a selection has changed
	 */
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {

		viewer.setInput(getModelFromSelection(selection));
	}

	/**
	 * Transforms the selection to the desired model, in our case
	 * PersistentTable
	 * 
	 * @param selection
	 */
	private PersistentTable getModelFromSelection(ISelection selection) {
		Object selPart = ((IStructuredSelection) selection).getFirstElement();
		Object model = (selPart instanceof EditPart) ? ((EditPart) selPart).getModel() : null;
		View view = (View) model;
		//graphical element, that has been selected
		switch (SqlmodelVisualIDRegistry.getVisualID(view)) {

		case PersistentTableEditPart.VISUAL_ID:
			return (PersistentTable) view.getElement();
		case PersistentTableNameEditPart.VISUAL_ID:
			return (PersistentTable) view.getElement();
		case ColumnEditPart.VISUAL_ID:
			return (PersistentTable) ((View) view.eContainer()).getElement();
		case PrimaryKeyEditPart.VISUAL_ID:
			return (PersistentTable) ((View) view.eContainer()).getElement();
		case WrapLabelEditPart.VISUAL_ID:
			return (PersistentTable) ((View) view.eContainer().eContainer()).getElement();
		case Column2EditPart.VISUAL_ID:
			return (PersistentTable) ((View) view.eContainer().eContainer().eContainer()).getElement();
		case PersistentTableColumnsCompartmentEditPart.VISUAL_ID:
			return (PersistentTable) ((View) view.eContainer()).getElement();
		case PrimaryKeyPrimaryKeyColumnsCompartmentEditPart.VISUAL_ID:
			return (PersistentTable) ((View) view.eContainer()).getElement();
		}
		return null;
	}
}